/*----------------------------------------------------------------------------*
 | This file is part of WinDEU, the port of DEU to Windows.                   |
 | WinDEU was created by the DEU team:                                        |
 |  Renaud Paquay, Raphael Quinet, Brendon Wyber and others...                |
 |                                                                            |
 | DEU is an open project: if you think that you can contribute, please join  |
 | the DEU team.  You will be credited for any code (or ideas) included in    |
 | the next version of the program.                                           |
 |                                                                            |
 | If you want to make any modifications and re-distribute them on your own,  |
 | you must follow the conditions of the WinDEU license. Read the file        |
 | LICENSE or README.TXT in the top directory.  If do not  have a copy of     |
 | these files, you can request them from any member of the DEU team, or by   |
 | mail: Raphael Quinet, Rue des Martyrs 9, B-4550 Nandrin (Belgium).         |
 |                                                                            |
 | This program comes with absolutely no warranty.  Use it at your own risks! |
 *----------------------------------------------------------------------------*

	Project WinDEU
	DEU team
	Jul-Dec 1994, Jan-Mar 1995

	FILE:         episdlg.cpp

	OVERVIEW
	========
	Source file for implementation of TEpisodeMissionDialog (TDialog).
*/

#include "common.h"
#pragma hdrstop

#ifndef __windeu_h
	#include "windeu.h"
#endif

#ifndef __wads_h
	#include "wads.h"
#endif

#ifndef __episdlg_h
	#include "episdlg.h"
#endif

#ifndef __OWL_RADIOBUT_H
	#include <owl\radiobut.h>
#endif

#ifndef __OWL_GROUPBOX_H
	#include <owl\groupbox.h>
#endif

#ifndef __OWL_CHECKBOX_H
	#include <owl\checkbox.h>
#endif

#ifndef __OWL_VALIDATE_H
	#include <owl\validate.h>
#endif



static LastEpisode = 1;
static LastMission = 1;
static LastMap     = 1;

//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TEpisodeMissionDialog, TDialog)
//{{TEpisodeMissionDialogRSP_TBL_BEGIN}}
	EV_BN_CLICKED(IDOK, CmOk),
	EV_BN_CLICKED(IDC_EM_DOOM2_CHECK, Doom2CheckClicked),
//{{TEpisodeMissionDialogRSP_TBL_END}}
END_RESPONSE_TABLE;


//{{TEpisodeMissionDialog Implementation}}


///////////////////////////////////////////////////////////
// TEpisodeMissionDialog
// ---------------------
//
TEpisodeMissionDialog::TEpisodeMissionDialog (TWindow* parent, char *LevelName,
											  int resId, TModule* module):
	TDialog(parent, resId, module)
{
	pLevelName = LevelName;

	if ( LastEpisode < 1 )	LastEpisode = 1;
	if ( LastEpisode > 3 )	LastEpisode = 3;
	if ( LastMission < 1 )	LastMission = 1;
	if ( LastMission > 9 )	LastMission = 9;
	if ( LastMap < 1 )		LastMap = 1;
	if ( LastMap > 32 )		LastMap = 32;

	pDoom2Check = new TCheckBox(this, IDC_EM_DOOM2_CHECK, 0);
	pDoom2Group = new TGroupBox(this, IDC_EM_DOOM2_GROUP);
	pDoom2MapEdit = new TEdit(this, IDC_EM_DOOM2_MAP);
	pDoom2MapEdit->SetValidator(new TRangeValidator(1,32));
	pDoom2MapStatic = new TStatic(this, IDC_EM_MAP_NUMBER);
	pMissionGroup = new TGroupBox(this, IDC_EM_MISSION_GROUP);
	pEpisodeGroup = new TGroupBox(this, IDC_EM_EPISODE_GROUP);

	for (SHORT e = 0 ; e < 3 ; e++)
		pEpisodeRadio[e] = new TRadioButton(this, IDC_EPISODE_FIRST+e);

	for (SHORT m = 0 ; m < 9 ; m++)
		pMissionRadio[m] = new TRadioButton(this, IDC_MISSION_FIRST+m);
}


///////////////////////////////////////////////////////////
// TEpisodeMissionDialog
// ---------------------
//
TEpisodeMissionDialog::~TEpisodeMissionDialog ()
{
	Destroy();
}


///////////////////////////////////////////////////////////
// TEpisodeMissionDialog
// ---------------------
//
void TEpisodeMissionDialog::SetupWindow ()
{
	TDialog::SetupWindow();
	::CenterWindow (this);

	// Sets episode 1 and mission 1
	pEpisodeRadio[LastEpisode - 1]->SetCheck (BF_CHECKED);
	pMissionRadio[LastMission - 1]->SetCheck (BF_CHECKED);

	char map[10];
	sprintf(map, "%d", LastMap);
	pDoom2MapEdit->SetText(map);

	if ( DoomVersion == 2 )
		pDoom2Check->SetCheck(BF_CHECKED);
	else
		pDoom2Check->SetCheck(BF_UNCHECKED);
	Doom2CheckClicked();
}


///////////////////////////////////////////////////////////
// TEpisodeMissionDialog
// ---------------------
//
void TEpisodeMissionDialog::CmOk ()
{
	if ( pDoom2Check->GetCheck() == BF_CHECKED )
	{
		char map[10];

		pDoom2MapEdit->GetText(map, 10);
		LastMap = atoi(map);
		sprintf (pLevelName, "MAP%02d", LastMap);
	}
	else
	{
		for (SHORT i = 0 ; i < 3 ; i++)
			// If radio button checked
			if ( pEpisodeRadio[i]->GetCheck() == BF_CHECKED )
			{
				LastEpisode = i + 1;
				break ;
			}

		for (i = 0 ; i < 9 ; i++)
			// If radio button checked
			if ( pMissionRadio[i]->GetCheck() == BF_CHECKED )
			{
				LastMission = i + 1 ;
				break ;
			}
		sprintf (pLevelName, "E%dM%d", LastEpisode, LastMission);
	}

	if ( FindMasterDir (MasterDir, pLevelName) == NULL )
	{
		Notify ("Invalid level number!");
		return;
	}

	TDialog::CmOk();
}


///////////////////////////////////////////////////////////
// TEpisodeMissionDialog
// ---------------------
//
void TEpisodeMissionDialog::Doom2CheckClicked ()
{
	// Enable DOOM2 map number
	if ( pDoom2Check->GetCheck() == BF_CHECKED )
	{
		for (int e = 0 ; e < 3; e++)
			pEpisodeRadio[e]->ShowWindow(SW_HIDE);
		for (int m = 0 ; m < 9; m++)
			pMissionRadio[m]->ShowWindow(SW_HIDE);
		pMissionGroup->ShowWindow(SW_HIDE);
		pEpisodeGroup->ShowWindow(SW_HIDE);

		pDoom2Group->ShowWindow(SW_SHOW);
		pDoom2MapEdit->ShowWindow(SW_SHOW);
		pDoom2MapStatic->ShowWindow(SW_SHOW);
	}
	// Enable DOOM1/HERETIC level number
	else
	{
		pDoom2Group->ShowWindow(SW_HIDE);
		pDoom2MapEdit->ShowWindow(SW_HIDE);
		pDoom2MapStatic->ShowWindow(SW_HIDE);

		for (int e = 0 ; e < 3; e++)
			pEpisodeRadio[e]->ShowWindow(SW_SHOW);
		for (int m = 0 ; m < 9; m++)
			pMissionRadio[m]->ShowWindow(SW_SHOW);
		pMissionGroup->ShowWindow(SW_SHOW);
		pEpisodeGroup->ShowWindow(SW_SHOW);
	}
}

