/*----------------------------------------------------------------------------*
 | This file is part of WinDEU, the port of DEU to Windows.                   |
 | WinDEU was created by the DEU team:                                        |
 |  Renaud Paquay, Raphael Quinet, Brendon Wyber and others...                |
 |                                                                            |
 | DEU is an open project: if you think that you can contribute, please join  |
 | the DEU team.  You will be credited for any code (or ideas) included in    |
 | the next version of the program.                                           |
 |                                                                            |
 | If you want to make any modifications and re-distribute them on your own,  |
 | you must follow the conditions of the WinDEU license. Read the file        |
 | LICENSE or README.TXT in the top directory.  If do not  have a copy of     |
 | these files, you can request them from any member of the DEU team, or by   |
 | mail: Raphael Quinet, Rue des Martyrs 9, B-4550 Nandrin (Belgium).         |
 |                                                                            |
 | This program comes with absolutely no warranty.  Use it at your own risks! |
 *----------------------------------------------------------------------------*

	Project WinDEU
	DEU team
	Jul-Dec 1994, Jan-Mar 1995

	FILE:         mainfram.cpp

	OVERVIEW
	========
	Source file for implementation of TMainFrame (TDecoratedFrame).
*/

#include "common.h"
#pragma hdrstop
#include "windeu.h"

#ifndef __OWL_STATUSBA_H
	#include <owl\statusba.h>
#endif

#ifndef __OWL_TEXTGADG_H
	#include <owl\textgadg.h>
#endif

#ifndef __mainfram_h
	#include "mainfram.h"
#endif

#ifndef __maincli_h
	#include "maincli.h"
#endif

#ifndef __editcli_h
	#include "editcli.h"
#endif

#include "owldef.rh"		// for IDI-SDIAPPLICATION
#include "mainmenu.rh"
#include "editmenu.rh"

//{{TMainFrame Implementation}}


/////////////////////////////////////////////////////////////////
// TMainFrame
// ----------
//
TMainFrame::TMainFrame (const char far *title, TModule* module):
	TDecoratedFrame(0, title, new TMainClient (0,""), TRUE, module)
{
	inEditor = FALSE ;

	// Override the default window style for TDecoratedFrame.
	Attr.Style |= WS_THICKFRAME | WS_CAPTION | WS_CLIPCHILDREN |
				  WS_MAXIMIZEBOX | WS_MINIMIZEBOX | WS_SYSMENU | WS_VISIBLE;

	// Assign ICON w/ this application.
	SetIcon (GetApplication(), IDI_SDIAPPLICATION);

	// Menu associated with window and accelerator table associated with table.
	SetMenuAndAccel (MENU_MAIN);

	// Create status bar
	// statusBar = new TStatusBar(this, TGadget::Recessed, TStatusBar::NumLock);
	statusBar = new TStatusBar(this, TGadget::Recessed, TStatusBar::ScrollLock);

	// Free memory display
	TTextGadget *tg1 = new TTextGadget(1, TGadget::Recessed,
									   TTextGadget::Left, 12) ;
	statusBar->Insert (*tg1) ;

	// Scale and grid display
	TTextGadget *tg2 = new TTextGadget(2, TGadget::Recessed,
									   TTextGadget::Left, 13) ;
	statusBar->Insert (*tg2) ;

	Insert(*statusBar, TDecoratedFrame::Bottom);
}


/////////////////////////////////////////////////////////////////
// TMainFrame
// ----------
//
TMainFrame::~TMainFrame ()
{
	Destroy();
}


/////////////////////////////////////////////////////////////////
// TMainFrame
// ----------
//
void TMainFrame::SetMenuAndAccel (TResId id)
{
	AssignMenu (id);
	Attr.AccelTable = id;
	LoadAcceleratorTable();
	DrawMenuBar();
}


/////////////////////////////////////////////////////////////////
// TMainFrame
// ----------
//
void TMainFrame::EditLevel (char *levelName, BOOL newLevel)
{
	TRACE ("TMainFrame::StartEditLevel: start");
	assert (inEditor == FALSE) ;

	// set new menu and accel.
	SetMenuAndAccel (MENU_EDITOR);

	// Hide client window
	TWindow *pClient = GetClientWindow () ;
	pClient->ShowWindow (SW_HIDE);

	// Hide status bar
	if ( ! InfoShown )
		ToggleStatusBar ();

	// Set editor client
	SetClientWindow (new TEditorClient (0, levelName, newLevel));

	// delete old client window
	// pClient->CloseWindow();
	delete pClient ;

	inEditor = TRUE ;
}


/////////////////////////////////////////////////////////////////
// TMainFrame
// ----------
//
BOOL TMainFrame::StopEditLevel ()
{
	TRACE ("TMainFrame::StopEditLevel: start");
	assert (inEditor == TRUE) ;
	TEditorClient *pClient =
		TYPESAFE_DOWNCAST(GetClientWindow (), TEditorClient);

	// Try to save editor changes
	if ( pClient->SaveChanges() == FALSE )
		return FALSE;

	// Set new menu and accel.
	SetMenuAndAccel (MENU_MAIN);

	// Hide editor client window
	pClient->ShowWindow (SW_HIDE);

	// Show status bar
	if ( ! InfoShown )
		ToggleStatusBar ();

	// Set new Main client window
	SetClientWindow (new TMainClient (0, ""));

	// Delete old client
	// pClient->CloseWindow();
	delete pClient ;

	// Clear status bar
	((TTextGadget *)(*GetStatusBar())[0])->SetText ("");
	((TTextGadget *)(*GetStatusBar())[1])->SetText ("");
	((TTextGadget *)(*GetStatusBar())[2])->SetText ("");

	inEditor = FALSE ;

	return TRUE;
}


/////////////////////////////////////////////////////////////////
// TMainFrame
// ----------
//
void TMainFrame::ToggleStatusBar ()
{
	EvCommand (IDW_STATUSBAR, 0, 0);
}

